
/******************************************************************************
 *
 * Str.C
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Headers
 *
 ******************************************************************************/

#include <stdio.h>
#include <ctype.h>

#ifdef VMS
#include <descrip.h>
#endif

#include "intrinsic.h"
#include "mem.h"
#include "str.h"

/******************************************************************************
 *
 * Local Constants and Macros
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Local Declarations
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Local Prototypes
 *
 ******************************************************************************/

/******************************************************************************
 *
 * StringCopy
 *
 ******************************************************************************/

#ifdef VMS
extern String StringCopy (String src)
#else
extern String StringCopy (src)

  String src;
#endif

{
  String x;

  /* allocate */
  x = (String) MemAlloc (strlen(src) + 1, NoInitialize);

  /* copy */
  strcpy (x, src);

  /* return it */
  return (x);
}

/******************************************************************************
 *
 * StringLeft
 *
 ******************************************************************************/

#ifdef VMS
extern void StringLeft (String src, int pos)
#else
extern void StringLeft (src, pos)

  String src;
  int    pos;
#endif

{
  /* validate */
  if ((pos < 0) || (pos > strlen(src))) return;

  /* truncate string */
  src[pos] = '\0';

  /* done */
  return;
}

/******************************************************************************
 *
 * StringRight
 *
 ******************************************************************************/

#ifdef VMS
extern void StringRight (String str, int pos)
#else
extern void StringRight (str, pos)

  String str;
  int    pos;
#endif

{
  Buffer temp;

  /* validate */
  if ((pos < 0) || (pos > strlen(str))) return;

  /* extract */
  strcpy (temp, &(str[pos]));

  /* copy back */
  strcpy (str, temp);

  /* done */
  return;
}

/******************************************************************************
 *
 * StringSubstr
 *
 ******************************************************************************/

#ifdef VMS
extern void StringSubstr (String str, int pos, int count)
#else
extern void StringSubstr (str, pos, count)

  String str;
  int    pos;
  int    count;
#endif

{
  Buffer temp;

  /* validate */
  if ((pos < 0) || (pos > strlen(str))) return;
  if ((pos + count) > strlen(str)) return;

  /* extract */
  strcpy (temp, &(str[pos]));

  /* truncate */
  temp[count] = '\0';

  /* copy back */
  strcpy (str, temp);

  /* done */
  return;
}

/******************************************************************************
 *
 * StringUppercase
 *
 ******************************************************************************/

#ifdef VMS
extern void StringUppercase (String str)
#else
extern void StringUppercase (str)

  String str;
#endif

{
  /* process */
  while (*str) {

    /* if lowercase, then uppercase it */
    if (islower(*str)) *str = toupper(*str);

    /* go to next character */
    str += 1;
  }

  /* done */
  return;
}

/******************************************************************************
 *
 * StringLowercase
 *
 ******************************************************************************/

#ifdef VMS
extern void StringLowercase (String str)
#else
extern void StringLowercase (str)

  String str;
#endif

{
  /* process */
  while (*str) {

    /* if uppercase, then lowercase it */
    if (isupper(*str)) *str = tolower(*str);

    /* go to next character */
    str += 1;
  }

  /* done */
  return;
}

/******************************************************************************
 *
 * StringDesc
 *
 ******************************************************************************/

#ifdef VMS
Pointer StringDesc (String src)

{
  static Buffer      temp;
  static $DESCRIPTOR (desc, temp);

  /* fill in descriptor */
  strcpy (temp, src);
  desc.dsc$w_length = strlen(src);

  /* return */
  return ((Pointer) (&desc));
}
#endif

/******************************************************************************
 *
 * StringAllocDesc
 *
 ******************************************************************************/

#ifdef VMS
Pointer StringAllocDesc (String str)

{
  Pointer     x;
  $DESCRIPTOR (desc, str);

  /* revise */
  desc.dsc$w_length = strlen(str);

  /* copy */
  x = (Pointer) MemAlloc (sizeof(desc), NoInitialize);
  MemCopy (x, &desc, sizeof(desc));

  /* done */
  return (x);
}
#endif

/******************************************************************************
 *
 * StringDeallocDesc
 *
 ******************************************************************************/

#ifdef VMS
extern void StringDeallocDesc (Pointer desc)

{
  /* free */
  MemDealloc (desc);

  /* done */
  return;
}
#endif

/******************************************************************************
 *
 * StringReplace
 *
 ******************************************************************************/

#ifdef VMS
extern void StringReplace (String str, int pos, int count, String replace)
#else
extern void StringReplace (str, pos, count, replace)

  String str;
  int    pos;
  int    count;
  String replace;
#endif

{
  Buffer temp;

  /* validate */
  if ((pos < 0) || (pos > strlen(str))) return;
  if ((pos + count) > strlen(str)) return;

  /* copy original */
  strcpy (temp, str);

  /* copy replacement */
  strcpy (&temp[pos], replace);

  /* put back remainder */
  strcpy (&temp[strlen(temp)], &str[pos + count]);

  /* copy back */
  strcpy (str, temp);

  /* done */
  return;
}
